;nyquist plug-in
;version 3
;type generate
;name "Quick Calculator"
;action "Calculating..."
;info "Quick Calculator v.0.2 By Steve Daulton.\nhttp://easyspacepro.com\n\nTo select the reverse function (eg. 'Linear to dB' vs 'dB to Linear')\nset the 'Direction of Conversion' to 'Inverse'.\n\nFor conversions that require more than one input value,\nseparate the values with spaces."

;control type "Type of Conversion" choice "dB to Linear,dB to %,BPM to Beat Length,MIDI to Hz,Hz to Period (seconds),Hz to Samples,Tempo Change to %,deg to rad,Add numbers,Multiply Numbers,Divide (a/b),Nyquist Code" 0 
;control d "Direction of Conversion" choice "Normal (forward),Inverse" 0
;control input "Enter data to evaluate" string ""  


;;; Get list from string
(defun lfs (x)
  (read (make-string-input-stream (format nil "(~a)" x))))

;;; Get number from string
(defun nfs (x)
  (read (make-string-input-stream x)))

#|
;; template for conversion function
(defun bpm (x dir)
   (case dir
      (0 (format NIL "~a~a" (nfs x) 
      (1 (format NIL "~a~a" (nfs x)
|#

;;; dB to linear
(defun db2l (x dir)
  (case dir 
    (0 (format NIL
          "~a dB = ~a (linear)"
          (nfs x) (db-to-linear (nfs x))))
    (T (format NIL
          "~a (linear) = ~a dB"
          (nfs x) (linear-to-db (nfs x))))))

;;; dB to percent
(defun db2p (x dir)
   (case dir 
      (0 (format NIL "~a dB = ~a %" (nfs x) (* 100 (db-to-linear (nfs x)))))
      (T (format NIL "~a % = ~a dB" (nfs x) (linear-to-db (/ (nfs x) 100.0))))))

;;; BPM to beat length
(defun bpm (x dir)
   (case dir
      (0 (format NIL
            "The beat length at ~a bpm~%= ~a seconds"
            (nfs x) (/ 60.0 (nfs x))))
      (1 (format NIL
            "A beat length of ~a seconds~%is equivalent to ~a bpm"
            (nfs x) (/ 60.0 (nfs x))))))

;;; Frequency (Hz) to period (seconds)
(defun Hz2p (x dir)
  (case dir
    (0 (format NIL
        "~a Hz has a period of ~a seconds"
        (nfs x) (/ (float (nfs x)))))
    (1 (format NIL
        "A wave with a cycle period of ~a seconds~% has a frequency of ~a Hz."
        (nfs x) (/ (float (nfs x)))))))

;;; Frequency (Hz) to period (samples)
(defun Hz2smp (x dir)
  (case dir
    (0 (format NIL
        "~a Hz has a cycle length of ~a samples"
        (nfs x) (* *sound-srate* (/ (float (nfs x))))))
    (1 (format NIL
        "A wave with a cycle length of ~a samples~% has a frequency of ~a Hz."
        (nfs x) (/  *sound-srate* (float (nfs x)))))))

;; Tempo Change to % (useful for 'Sliding Time Scale / Pitch Shift'
(defun tempo2pc (x dir)
  (setq x (lfs x))
  (case dir
    (0 (if (/= (length x) 2)
        (format nil "'Tempo Change to %' requires~%both the initial tempo~%~
          and final tempo (bpm) to be entered.")
        (let ((out (* 100.0
                    (/ (- (second x)(first x))
                      (float (first x))))))
          (format NIL "Tempo change from~%~a bpm to ~a bpm is ~a %." 
            (first x) (second x) out))))
    (t (format nil "'Tempo Change to %'~%does not have an 'Inverse' version."))))

;;; MIDI note to Frequency (Hz)
(defun mid2Hz (x dir)
  (case dir
    (0 (format NIL
        "Midi note number ~a has a frequency of ~a Hz"
        (nfs x)(step-to-hz (nfs x))))
    (1(setq cent (round (* 100 (- (hz-to-step (nfs x))(round (hz-to-step (nfs x)))))))
      (cond
        ((>  cent 0)
          (format NIL
            "~a Hz is MIDI note number ~a (+ ~a cents)"
            (nfs x)(round (hz-to-step (nfs x))) cent))
        ((<  cent 0)
          (format NIL
          "~a Hz is MIDI note number ~a (- ~a cents)"
          (nfs x)(round (hz-to-step (nfs x)))(abs cent)))
        (T 
          (format NIL
            "~a Hz is MIDI note number ~a"
            (nfs x)(round (hz-to-step (nfs x)))))))))

;;; Degrees to radians
(defun d2r (x dir)
  (case dir
    (0 (format NIL
          "~a degrees = ~a radians"
          (nfs x) (* (nfs x)(/ pi 180.0))))
    (1 (format NIL
          "~a radians = ~a degrees"
          (nfs x) (* (nfs x)(/ 180.0 pi))))))

;;; Add
(defun add (x)
  (setq x (lfs x))
  (setq out (first x))
  (setq str (format NIL "~a" out))   
  (strcat
    (do ((i 1 (setq i (1+ i))))
        ((= i (length x)) str)
      (setq str (strcat str " + " (format NIL "~a" (nth i x))))
      (setq out (+ out (nth i x))))
    (format NIL " = ~a" out)))

;;; Multiply
(defun times (x)
  (setq x (lfs x))
  (setq out (first x))
  (setq str (format NIL "~a" out))
  (strcat
    (do ((i 1 (setq i (1+ i))))
        ((= i (length x)) str)
      (setq str (strcat str " x " (format NIL "~a" (nth i x))))
      (setq out (* out (nth i x))))
    (format NIL " = ~a" out)))

;;; Division
(defun divide (x)
  (setq x (lfs x))
  (setq out (first x))
  (setq str (format NIL "~a" out))
  (strcat
    (do ((i 1 (setq i (1+ i))))
        ((= i (length x)) str)
      (setq str (strcat str " / " (format NIL "~a" (nth i x))))
      (setq out (/ out (float (nth i x)))))
    (format NIL " = ~a" out)))

;;; Script
(defun nscript (x)
  (format NIL "Input:~%~a~%~%Output:~%~a" x
  (eval (read (make-string-input-stream (format NIL "~a" x))))))


;;; Select function
(case type
  (0 (db2l input d))
  (1 (db2p input d))
  (2 (bpm input d))
  (3 (mid2Hz input d))
  (4 (Hz2p input d))
  (5 (Hz2smp input d))
  (6 (tempo2pc input d))
  (7 (d2r input d))
  (8 (add input))
  (9 (times input))
  (10 (divide input))
  (11 (nscript input))
  (T (print "Not yet implemented")))
